@@ -1,7 +1,7 @@
 /* codablock.c - Handles Codablock-F */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2016-2022 Harald Oehlmann
+    Copyright (C) 2016-2023 Harald Oehlmann
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -555,7 +555,7 @@ INTERNAL int codablockf(struct zint_symbol *symbol, unsigned char source[], int
     /* option1: rows <= 0: automatic, 1..44 */
     rows = symbol->option_1;
     if (rows == 1) {
-        error_number = code128(symbol, source, length); /* Only returns errors, not warnings */
+        error_number = code128(symbol, source, length);
         if (error_number < ZINT_ERROR) {
             symbol->output_options |= BARCODE_BIND;
             if (symbol->border_width == 0) { /* Allow override if non-zero */
@@ -565,7 +565,11 @@ INTERNAL int codablockf(struct zint_symbol *symbol, unsigned char source[], int
             if (symbol->output_options & COMPLIANT_HEIGHT) {
                 /* AIM ISS-X-24 Section 4.6.1 minimum row height 8X (for compatibility with CODABLOCKF, not specced
                    for CODE128) */
-                error_number = set_height(symbol, 8.0f, 10.0f, 0.0f, 0 /*no_errtxt*/);
+                if (error_number == 0) {
+                    error_number = set_height(symbol, 8.0f, 10.0f, 0.0f, 0 /*no_errtxt*/);
+                } else {
+                    (void) set_height(symbol, 8.0f, 10.0f, 0.0f, 1 /*no_errtxt*/);
+                }
             } else {
                 (void) set_height(symbol, 0.0f, 5.0f, 0.0f, 1 /*no_errtxt*/);
             }
@@ -582,6 +586,9 @@ INTERNAL int codablockf(struct zint_symbol *symbol, unsigned char source[], int
         strcpy(symbol->errtxt, "411: Columns parameter not in 0, 9..67");
         return ZINT_ERROR_INVALID_OPTION;
     }
+    if (columns < 0) { /* Protect against negative overflow (ticket #300 (#9) Andre Maute) */
+        columns = 0;
+    }
 
     data = (unsigned char *) z_alloca(length * 2 + 1);
 
@@ -645,10 +652,10 @@ INTERNAL int codablockf(struct zint_symbol *symbol, unsigned char source[], int
 
     if (symbol->debug & ZINT_DEBUG_PRINT) { /* start a new level of local variables */
         int DPos;
-        printf("\nData:");
+        fputs("\nData:", stdout);
         for (DPos = 0; DPos < dataLength; DPos++)
             fputc(data[DPos], stdout);
-        printf("\n Set:");
+        fputs("\n Set:", stdout);
         for (DPos = 0; DPos < dataLength; DPos++) {
             switch (pSet[DPos] & (CodeA + CodeB + CodeC)) {
             case CodeA: fputc('A', stdout); break;
@@ -657,16 +664,16 @@ INTERNAL int codablockf(struct zint_symbol *symbol, unsigned char source[], int
             default: fputc('.', stdout); break;
             }
         }
-        printf("\nFNC1:");
+        fputs("\nFNC1:", stdout);
         for (DPos = 0; DPos < dataLength; DPos++)
             fputc((pSet[DPos] & CodeFNC1) == 0 ? '.' : 'X', stdout);
-        printf("\n END:");
+        fputs("\n END:", stdout);
         for (DPos = 0; DPos < dataLength; DPos++)
             fputc((pSet[DPos] & CEnd) == 0 ? '.' : 'X', stdout);
-        printf("\nShif:");
+        fputs("\nShif:", stdout);
         for (DPos = 0; DPos < dataLength; DPos++)
             fputc((pSet[DPos] & CShift) == 0 ? '.' : 'X', stdout);
-        printf("\nFILL:");
+        fputs("\nFILL:", stdout);
         for (DPos = 0; DPos < dataLength; DPos++)
             fputc((pSet[DPos] & CFill) == 0 ? '.' : 'X', stdout);
         fputc('\n', stdout);
@@ -831,14 +838,14 @@ INTERNAL int codablockf(struct zint_symbol *symbol, unsigned char source[], int
     if (symbol->debug & ZINT_DEBUG_PRINT) {
         /* Dump the output to the screen
          */
-        printf("\nCode 128 Code Numbers:\n");
+        fputs("\nCode 128 Code Numbers:\n", stdout);
         { /* start a new level of local variables */
             int DPos, DPos2;
             for (DPos = 0; DPos < rows; DPos++) {
                 for (DPos2 = 0; DPos2 < columns; DPos2++) {
                     printf("%3d ", (int) (pOutput[DPos * columns + DPos2]));
                 }
-                printf("\n");
+                fputc('\n', stdout);
             }
         }
         printf("rows=%d columns=%d (%d data) fillings=%d\n", rows, columns, columns - 5, fillings);
