@@ -1,7 +1,7 @@
 /* code.c - Handles Code 11, 39, 39+, 93, PZN, Channel and VIN */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2008-2022 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2008-2023 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -132,8 +132,9 @@ INTERNAL int code11(struct zint_symbol *symbol, unsigned char source[], int leng
 
     int i;
     int h, c_digit, c_weight, c_count, k_digit, k_weight, k_count;
-    int weight[122], error_number = 0;
-    char dest[750]; /* 6 + 121 * 6 + 2 * 6 + 5 + 1 == 750 */
+    int weight[141]; /* 140 + 1 extra for 1st check */
+    char dest[864]; /* 6 + 140 * 6 + 2 * 6 + 5 + 1 = 864 */
+    int error_number = 0;
     char *d = dest;
     int num_check_digits;
     char checkstr[3] = {0};
@@ -142,8 +143,8 @@ INTERNAL int code11(struct zint_symbol *symbol, unsigned char source[], int leng
     /* Suppresses clang-tidy clang-analyzer-core.UndefinedBinaryOperatorResult warning */
     assert(length > 0);
 
-    if (length > 121) {
-        strcpy(symbol->errtxt, "320: Input too long (121 character maximum)");
+    if (length > 140) { /* 8 (Start) + 140 * 8 + 2 * 8 (Check) + 7 (Stop) = 1151 */
+        strcpy(symbol->errtxt, "320: Input too long (140 character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
     if (!is_sane(SODIUM_MNS_F, source, length)) {
@@ -241,14 +242,14 @@ INTERNAL int code39(struct zint_symbol *symbol, unsigned char source[], int leng
     int i;
     int counter;
     int error_number = 0;
-    int posns[85];
-    char dest[880]; /* 10 (Start) + 85 * 10 + 10 (Check) + 9 (Stop) + 1 = 880 */
+    int posns[86];
+    char dest[890]; /* 10 (Start) + 86 * 10 + 10 (Check) + 9 (Stop) + 1 = 890 */
     char *d = dest;
     char localstr[2] = {0};
 
     counter = 0;
 
-    if ((symbol->option_2 < 0) || (symbol->option_2 > 1)) {
+    if ((symbol->option_2 < 0) || (symbol->option_2 > 2)) {
         symbol->option_2 = 0;
     }
 
@@ -257,17 +258,17 @@ INTERNAL int code39(struct zint_symbol *symbol, unsigned char source[], int leng
         strcpy(symbol->errtxt, "322: Input too long (30 character maximum)");
         return ZINT_ERROR_TOO_LONG;
     /* Prevent encoded_data out-of-bounds >= 143 for BARCODE_HIBC_39 due to wider 'wide' bars */
-    } else if ((symbol->symbology == BARCODE_HIBC_39) && (length > 69)) {
+    } else if ((symbol->symbology == BARCODE_HIBC_39) && (length > 70)) { /* 16 (Start) + 70*16 + 15 (Stop) = 1151 */
         /* Note use 319 (2of5 range) as 340 taken by CODE128 */
-        strcpy(symbol->errtxt, "319: Input too long (67 character maximum)"); /* 69 less '+' and check */
+        strcpy(symbol->errtxt, "319: Input too long (68 character maximum)"); /* 70 less '+' and check */
         return ZINT_ERROR_TOO_LONG;
-    } else if (length > 85) {
-        strcpy(symbol->errtxt, "323: Input too long (85 character maximum)");
+    } else if (length > 86) { /* 13 (Start) + 86*13 + 12 (Stop) = 1143 */
+        strcpy(symbol->errtxt, "323: Input too long (86 character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
 
     to_upper(source, length);
-    if (!is_sane_lookup(SILVER, 43, source, length, posns)) {
+    if (!is_sane_lookup(SILVER, 43 /* Up to "%" */, source, length, posns)) {
         strcpy(symbol->errtxt, "324: Invalid character in data (alphanumerics, space and \"-.$/+%\" only)");
         return ZINT_ERROR_INVALID_DATA;
     }
@@ -281,7 +282,7 @@ INTERNAL int code39(struct zint_symbol *symbol, unsigned char source[], int leng
         counter += posns[i];
     }
 
-    if (symbol->option_2 == 1) {
+    if (symbol->option_2 == 1 || symbol->option_2 == 2) { /* Visible or hidden check digit */
 
         char check_digit;
         counter %= 43;
@@ -294,8 +295,11 @@ INTERNAL int code39(struct zint_symbol *symbol, unsigned char source[], int leng
             check_digit = '_';
         }
 
-        localstr[0] = check_digit;
-        localstr[1] = '\0';
+        if (symbol->option_2 == 1) { /* Visible check digit */
+            localstr[0] = check_digit;
+            localstr[1] = '\0';
+        }
+        if (symbol->debug & ZINT_DEBUG_PRINT) printf("Check digit: %c\n", check_digit);
     }
 
     /* Stop character */
@@ -303,7 +307,7 @@ INTERNAL int code39(struct zint_symbol *symbol, unsigned char source[], int leng
     d += 9;
 
     if ((symbol->symbology == BARCODE_LOGMARS) || (symbol->symbology == BARCODE_HIBC_39)) {
-        /* LOGMARS uses wider 'wide' bars than normal Code 39 */
+        /* LOGMARS and HIBC use wider 'wide' bars than normal Code 39 */
         counter = d - dest;
         for (i = 0; i < counter; i++) {
             if (dest[i] == '2') {
@@ -439,13 +443,14 @@ INTERNAL int pzn(struct zint_symbol *symbol, unsigned char source[], int length)
 /* Extended Code 39 - ISO/IEC 16388:2007 Annex A */
 INTERNAL int excode39(struct zint_symbol *symbol, unsigned char source[], int length) {
 
-    unsigned char buffer[85 * 2 + 1] = {0};
+    unsigned char buffer[86 * 2 + 1] = {0};
     unsigned char *b = buffer;
+    unsigned char check_digit = '\0';
     int i;
     int error_number;
 
-    if (length > 85) {
-        strcpy(symbol->errtxt, "328: Input too long (85 character maximum)");
+    if (length > 86) {
+        strcpy(symbol->errtxt, "328: Input too long (86 character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
 
@@ -459,8 +464,8 @@ INTERNAL int excode39(struct zint_symbol *symbol, unsigned char source[], int le
         memcpy(b, EC39Ctrl[source[i]], 2);
         b += EC39Ctrl[source[i]][1] ? 2 : 1;
     }
-    if (b - buffer > 85) {
-        strcpy(symbol->errtxt, "317: Expanded input too long (85 symbol character maximum)");
+    if (b - buffer > 86) {
+        strcpy(symbol->errtxt, "317: Expanded input too long (86 symbol character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
     *b = '\0';
@@ -468,9 +473,22 @@ INTERNAL int excode39(struct zint_symbol *symbol, unsigned char source[], int le
     /* Then sends the buffer to the C39 function */
     error_number = code39(symbol, buffer, b - buffer);
 
+    /* Save visible check digit */
+    if (symbol->option_2 == 1) {
+        const int len = (int) ustrlen(symbol->text);
+        if (len > 0) {
+            check_digit = symbol->text[len - 1];
+        }
+    }
+
+    /* Copy over source to HRT, subbing space for unprintables */
     for (i = 0; i < length; i++)
         symbol->text[i] = source[i] >= ' ' && source[i] != 0x7F ? source[i] : ' ';
-    symbol->text[length] = '\0'; /* Chops off check digit */
+
+    if (check_digit) {
+        symbol->text[i++] = check_digit;
+    }
+    symbol->text[i] = '\0';
 
     return error_number;
 }
@@ -484,17 +502,17 @@ INTERNAL int code93(struct zint_symbol *symbol, unsigned char source[], int leng
 
     int i;
     int h, weight, c, k, error_number = 0;
-    int values[110]; /* 107 + 2 (Checks) */
-    char buffer[216]; /* 107*2 (107 full ASCII) + 1 = 215 */
+    int values[125]; /* 123 + 2 (Checks) */
+    char buffer[247]; /* 123*2 (123 full ASCII) + 1 = 247 */
     char *b = buffer;
-    char dest[668]; /* 6 (Start) + 107*6 + 2*6 (Checks) + 7 (Stop) + 1 (NUL) = 668 */
+    char dest[764]; /* 6 (Start) + 123*6 + 2*6 (Checks) + 7 (Stop) + 1 (NUL) = 764 */
     char *d = dest;
 
     /* Suppresses clang-tidy clang-analyzer-core.CallAndMessage warning */
     assert(length > 0);
 
-    if (length > 107) { /* 9 (Start) + 107*9 + 2*9 (Checks) + 10 (Stop) == 1000 */
-        strcpy(symbol->errtxt, "330: Input too long (107 character maximum)");
+    if (length > 123) { /* 9 (Start) + 123*9 + 2*9 (Checks) + 10 (Stop) = 1144 */
+        strcpy(symbol->errtxt, "330: Input too long (123 character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
 
@@ -512,8 +530,8 @@ INTERNAL int code93(struct zint_symbol *symbol, unsigned char source[], int leng
 
     /* Now we can check the true length of the barcode */
     h = b - buffer;
-    if (h > 107) {
-        strcpy(symbol->errtxt, "332: Expanded input too long (107 symbol character maximum)");
+    if (h > 123) {
+        strcpy(symbol->errtxt, "332: Expanded input too long (123 symbol character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
 
@@ -600,11 +618,11 @@ static void channel_generate_precalc(int channels, long value, int mod, int last
             int smax[7]) {
     int i;
     if (value == mod) printf("static channel_precalc channel_precalcs%d[] = {\n", channels);
-    printf("    { %7ld, {", value); for (i = 0; i < 8; i++) printf(" %d,", B[i]); printf(" },");
-    printf(" {"); for (i = 0; i < 8; i++) printf(" %d,", S[i]); printf(" },");
-    printf(" {"); for (i = 0; i < 7; i++) printf(" %d,", bmax[i]); printf(" },");
-    printf(" {"); for (i = 0; i < 7; i++) printf(" %d,", smax[i]); printf(" }, },\n");
-    if (value == last) printf("};\n");
+    printf("    { %7ld, {", value); for (i = 0; i < 8; i++) printf(" %d,", B[i]); fputs(" },", stdout);
+    fputs(" {", stdout); for (i = 0; i < 8; i++) printf(" %d,", S[i]); fputs(" },", stdout);
+    fputs(" {", stdout); for (i = 0; i < 7; i++) printf(" %d,", bmax[i]); fputs(" },", stdout);
+    fputs(" {", stdout); for (i = 0; i < 7; i++) printf(" %d,", smax[i]); fputs(" }, },\n", stdout);
+    if (value == last) fputs("};\n", stdout);
 }
 #else
 #include "channel_precalcs.h"
