@@ -1,7 +1,7 @@
 /* code1.h - Lookup info for USS Code One */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2009-2022 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2009-2023 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -97,12 +97,5 @@ static const unsigned short c1_grid_height[] = {
     5, 7, 10, 15, 21, 30, 46, 68
 };
 
-#define C1_ASCII    1
-#define C1_C40      2
-#define C1_DECIMAL  3
-#define C1_TEXT     4
-#define C1_EDI      5
-#define C1_BYTE     6
-
 /* vim: set ts=4 sw=4 et : */
 #endif /* Z_CODE1_H */
