@@ -1,7 +1,7 @@
 /*  hanxin.c - Han Xin Code */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2009-2022 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2009-2023 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -308,7 +308,7 @@ static int hx_in_numeric(const unsigned int ddata[], const int length, const int
     }
 
     /* Attempt to calculate the average 'cost' of using numeric mode in number of bits (times HX_MULT) */
-    for (i = in_posn; i < length && i < in_posn + 4 && z_isdigit(ddata[i]); i++);
+    for (i = in_posn; i < length && i < in_posn + 3 && z_isdigit(ddata[i]); i++);
 
     digit_cnt = i - in_posn;
 
@@ -548,7 +548,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                 bp = bin_append_posn(1, 4, binary, bp);
 
                 if (debug_print) {
-                    printf("Numeric\n");
+                    printf("Numeric (N%d): ", block_length);
                 }
 
                 count = 0; /* Suppress gcc -Wmaybe-uninitialized */
@@ -574,7 +574,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                     bp = bin_append_posn(encoding_value, 10, binary, bp);
 
                     if (debug_print) {
-                        printf("0x%3x (%d)", encoding_value, encoding_value);
+                        printf(" 0x%3x(%d)", encoding_value, encoding_value);
                     }
 
                     i += count;
@@ -604,7 +604,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                 bp = bin_append_posn(2, 4, binary, bp);
 
                 if (debug_print) {
-                    printf("Text\n");
+                    printf("Text (T%d):", block_length);
                 }
 
                 submode = 1;
@@ -618,7 +618,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                         bp = bin_append_posn(62, 6, binary, bp);
                         submode = hx_getsubmode(ddata[i + position]);
                         if (debug_print) {
-                            printf("SWITCH ");
+                            fputs(" SWITCH", stdout);
                         }
                     }
 
@@ -631,7 +631,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                     bp = bin_append_posn(encoding_value, 6, binary, bp);
 
                     if (debug_print) {
-                        printf("%.2x [ASC %.2x] ", encoding_value, ddata[i + position]);
+                        printf(" %.2x[ASC %.2x]", encoding_value, ddata[i + position]);
                     }
                     i++;
                 }
@@ -640,7 +640,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                 bp = bin_append_posn(63, 6, binary, bp);
 
                 if (debug_print) {
-                    printf("\n");
+                    fputs("\n", stdout);
                 }
                 break;
             case 'b':
@@ -652,7 +652,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                 bp = bin_append_posn(block_length + double_byte, 13, binary, bp);
 
                 if (debug_print) {
-                    printf("Binary Mode (%d):", block_length + double_byte);
+                    printf("Binary Mode (B%d):", block_length + double_byte);
                 }
 
                 i = 0;
@@ -670,7 +670,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                 }
 
                 if (debug_print) {
-                    printf("\n");
+                    fputs("\n", stdout);
                 }
                 break;
             case '1':
@@ -681,7 +681,8 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                 }
 
                 if (debug_print) {
-                    printf("Region One%s\n", position == 0 || mode[position - 1] != '2' ? "" : " (NO indicator)" );
+                    printf("Region One%s H(1)%d:",
+                        position == 0 || mode[position - 1] != '2' ? "" : " (NO indicator)", block_length);
                 }
 
                 i = 0;
@@ -706,7 +707,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                     }
 
                     if (debug_print) {
-                        printf("%.3x [GB %.4x] ", glyph, ddata[i + position]);
+                        printf(" %.3x[GB %.4x]", glyph, ddata[i + position]);
                     }
 
                     bp = bin_append_posn(glyph, 12, binary, bp);
@@ -718,7 +719,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                                     ? 4095 : 4094, 12, binary, bp);
 
                 if (debug_print) {
-                    printf("(TERM %x)\n", position + block_length == length || mode[position + block_length] != '2'
+                    printf(" (TERM %x)\n", position + block_length == length || mode[position + block_length] != '2'
                             ? 4095 : 4094);
                 }
 
@@ -731,7 +732,8 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                 }
 
                 if (debug_print) {
-                    printf("Region Two%s\n", position == 0 || mode[position - 1] != '1' ? "" : " (NO indicator)" );
+                    printf("Region Two%s H(2)%d:",
+                            position == 0 || mode[position - 1] != '1' ? "" : " (NO indicator)", block_length);
                 }
 
                 i = 0;
@@ -743,7 +745,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                     glyph = (0x5e * (first_byte - 0xd8)) + (second_byte - 0xa1);
 
                     if (debug_print) {
-                        printf("%.3x [GB %.4x] ", glyph, ddata[i + position]);
+                        printf(" %.3x[GB %.4x]", glyph, ddata[i + position]);
                     }
 
                     bp = bin_append_posn(glyph, 12, binary, bp);
@@ -755,7 +757,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                                     ? 4095 : 4094, 12, binary, bp);
 
                 if (debug_print) {
-                    printf("(TERM %x)\n", position + block_length == length || mode[position + block_length] != '1'
+                    printf(" (TERM %x)\n", position + block_length == length || mode[position + block_length] != '1'
                             ? 4095 : 4094);
                 }
 
@@ -766,7 +768,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                 bp = bin_append_posn(6, 4, binary, bp);
 
                 if (debug_print) {
-                    printf("Double byte\n");
+                    printf("Double byte (H(d)%d):", block_length);
                 }
 
                 i = 0;
@@ -795,13 +797,13 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                    - confirmed by Wang Yi */
 
                 if (debug_print) {
-                    printf("\n");
+                    fputc('\n', stdout);
                 }
                 break;
             case 'f':
                 /* Four-byte encoding */
                 if (debug_print) {
-                    printf("Four byte\n");
+                    printf("Four byte (H(f)%d):", block_length);
                 }
 
                 i = 0;
@@ -820,7 +822,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                             (0x0a * (third_byte - 0x81)) + (fourth_byte - 0x30);
 
                     if (debug_print) {
-                        printf("%d ", glyph);
+                        printf(" %d", glyph);
                     }
 
                     bp = bin_append_posn(glyph, 21, binary, bp);
@@ -830,7 +832,7 @@ static void hx_calculate_binary(char binary[], const char mode[], const unsigned
                 /* No terminator */
 
                 if (debug_print) {
-                    printf("\n");
+                    fputc('\n', stdout);
                 }
                 break;
         }
@@ -1454,7 +1456,7 @@ static void hx_apply_bitmask(unsigned char *grid, const int size, const int vers
         if (!user_mask) {
             for (pattern = 0; pattern < 4; pattern++) printf(" %d:%d", pattern, penalty[pattern]);
         }
-        printf("\n");
+        fputc('\n', stdout);
     }
 
     /* Apply mask */
@@ -1554,7 +1556,7 @@ INTERNAL int hanxin(struct zint_symbol *symbol, struct zint_seg segs[], const in
         codewords++;
     }
     if (debug_print) {
-        printf("Num. of codewords: %d\n", codewords);
+        printf("Num. of codewords: %d (%d padbits)\n", codewords, bin_len & 0x07);
     }
 
     version = 85;
@@ -1645,11 +1647,11 @@ INTERNAL int hanxin(struct zint_symbol *symbol, struct zint_seg segs[], const in
     }
 
     if (debug_print) {
-        printf("Datastream (%d): ", data_codewords);
+        printf("Datastream (%d):", data_codewords);
         for (i = 0; i < data_codewords; i++) {
-            printf("%.2x ", datastream[i]);
+            printf(" %.2x", datastream[i]);
         }
-        printf("\n");
+        fputc('\n', stdout);
     }
 #ifdef ZINT_TEST
     if (symbol->debug & ZINT_DEBUG_TEST) debug_test_codeword_dump(symbol, datastream, data_codewords);
@@ -1660,11 +1662,11 @@ INTERNAL int hanxin(struct zint_symbol *symbol, struct zint_seg segs[], const in
     hx_add_ecc(fullstream, datastream, data_codewords, version, ecc_level);
 
     if (debug_print) {
-        printf("Fullstream (%d): ", hx_total_codewords[version - 1]);
+        printf("Fullstream (%d):", hx_total_codewords[version - 1]);
         for (i = 0; i < hx_total_codewords[version - 1]; i++) {
-            printf("%.2x ", fullstream[i]);
+            printf(" %.2x", fullstream[i]);
         }
-        printf("\n");
+        fputc('\n', stdout);
     }
 
     hx_make_picket_fence(fullstream, picket_fence, hx_total_codewords[version - 1]);
