@@ -1,7 +1,7 @@
 /* imail.c - Handles Intelligent Mail (aka OneCode) for USPS */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2008-2022 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2008-2023 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -33,7 +33,6 @@
 /*  The function "USPS_MSB_Math_CRC11GenerateFrameCheckSequence"
     is Copyright (C) 2006 United States Postal Service */
 
-#include <stdio.h>
 #include "common.h"
 #include "large.h"
 
@@ -247,9 +246,10 @@ INTERNAL int usps_imail(struct zint_symbol *symbol, unsigned char source[], int
     char data_pattern[200];
     int error_number = 0;
     int i, j, read;
-    char zip[35], tracker[35], temp[2];
-    large_int accum;
-    large_int byte_array_reg;
+    char tracker[33] = {0}; /* Zero to prevent false warning from clang-tidy */
+    char zip[33], temp[2];
+    large_uint accum;
+    large_uint byte_array_reg;
     unsigned char byte_array[13];
     unsigned short usps_crc;
     unsigned int codeword[10];
@@ -266,11 +266,9 @@ INTERNAL int usps_imail(struct zint_symbol *symbol, unsigned char source[], int
         return ZINT_ERROR_INVALID_DATA;
     }
 
-    strcpy(zip, "");
-    strcpy(tracker, "");
-
     /* separate the tracking code from the routing code */
 
+    zip[0] = '\0';
     read = 0;
     j = 0;
     for (i = 0; i < length; i++) {
@@ -342,7 +340,7 @@ INTERNAL int usps_imail(struct zint_symbol *symbol, unsigned char source[], int
 
     /* and then the rest */
 
-    for (read = 2, len = (int) strlen(tracker); read < len; read++) {
+    for (read = 2; read < 20; read++) {
 
         large_mul_u64(&accum, 10);
         large_add_u64(&accum, ctoi(tracker[read]));
@@ -407,7 +405,7 @@ INTERNAL int usps_imail(struct zint_symbol *symbol, unsigned char source[], int
         }
     }
 
-    strcpy(data_pattern, "");
+    data_pattern[0] = '\0';
     temp[1] = '\0';
     for (i = 0; i < 65; i++) {
         j = 0;
