@@ -1,7 +1,7 @@
 /*  library.c - external functions of libzint */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2009-2022 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2009-2023 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -38,6 +38,7 @@
 #include "common.h"
 #include "eci.h"
 #include "gs1.h"
+#include "output.h"
 #include "zfiletypes.h"
 
 /* It's assumed that int is at least 32 bits, the following will compile-time fail if not
@@ -49,12 +50,8 @@ typedef char static_assert_int32_is_32bits[sizeof(int32_t) * CHAR_BIT != 32 ? -1
 typedef char static_assert_uint32_is_32bits[sizeof(uint32_t) * CHAR_BIT != 32 ? -1 : 1];
 typedef char static_assert_uint64_at_least_64bits[sizeof(uint64_t) * CHAR_BIT < 64 ? -1 : 1];
 
-/* Create and initialize a symbol structure */
-struct zint_symbol *ZBarcode_Create(void) {
-    struct zint_symbol *symbol;
-
-    symbol = (struct zint_symbol *) calloc(1, sizeof(*symbol));
-    if (!symbol) return NULL;
+/* Set `symbol` to defaults (does not zeroize) */
+static void set_symbol_defaults(struct zint_symbol *symbol) {
 
     symbol->symbology = BARCODE_CODE128;
     symbol->scale = 1.0f;
@@ -69,15 +66,25 @@ struct zint_symbol *ZBarcode_Create(void) {
 #endif
     symbol->option_1 = -1;
     symbol->show_hrt = 1; /* Show human readable text */
-    symbol->fontsize = 8;
     symbol->input_mode = DATA_MODE;
     symbol->eci = 0; /* Default 0 uses ECI 3 */
     symbol->dot_size = 4.0f / 5.0f;
+    symbol->text_gap = 1.0f;
     symbol->guard_descent = 5.0f;
     symbol->warn_level = WARN_DEFAULT;
     symbol->bitmap = NULL;
     symbol->alphamap = NULL;
     symbol->vector = NULL;
+}
+
+/* Create and initialize a symbol structure */
+struct zint_symbol *ZBarcode_Create(void) {
+    struct zint_symbol *symbol;
+
+    symbol = (struct zint_symbol *) calloc(1, sizeof(*symbol));
+    if (!symbol) return NULL;
+
+    set_symbol_defaults(symbol);
 
     return symbol;
 }
@@ -108,12 +115,27 @@ void ZBarcode_Clear(struct zint_symbol *symbol) {
     }
     symbol->bitmap_width = 0;
     symbol->bitmap_height = 0;
-    symbol->bitmap_byte_length = 0;
 
     /* If there is a rendered version, ensure its memory is released */
     vector_free(symbol);
 }
 
+/* Free any output buffers that may have been created and reset all fields to defaults */
+void ZBarcode_Reset(struct zint_symbol *symbol) {
+    if (!symbol) return;
+
+    if (symbol->bitmap != NULL) {
+        free(symbol->bitmap);
+    }
+    if (symbol->alphamap != NULL) {
+        free(symbol->alphamap);
+    }
+    vector_free(symbol);
+
+    memset(symbol, 0, sizeof(*symbol));
+    set_symbol_defaults(symbol);
+}
+
 /* Free a symbol structure, including any output buffers */
 void ZBarcode_Delete(struct zint_symbol *symbol) {
     if (!symbol) return;
@@ -216,9 +238,7 @@ INTERNAL int plot_vector(struct zint_symbol *symbol, int rotate_angle, int file_
 static int error_tag(struct zint_symbol *symbol, int error_number, const char *error_string) {
 
     if (error_number != 0) {
-        static const char error_fmt[] = "Error %.93s"; /* Truncate if too long */
-        static const char warn_fmt[] = "Warning %.91s"; /* Truncate if too long */
-        const char *fmt = error_number >= ZINT_ERROR ? error_fmt : warn_fmt;
+        const char *const error_arg = error_string ? error_string : symbol->errtxt;
         char error_buffer[100];
 
         if (error_number < ZINT_ERROR && symbol->warn_level == WARN_FAIL_ALL) {
@@ -227,12 +247,20 @@ static int error_tag(struct zint_symbol *symbol, int error_number, const char *e
                 error_number = ZINT_ERROR_NONCOMPLIANT;
             } else if (error_number == ZINT_WARN_USES_ECI) {
                 error_number = ZINT_ERROR_USES_ECI;
-            } else { /* ZINT_WARN_INVALID_OPTION */
+            } else if (error_number == ZINT_WARN_INVALID_OPTION) {
                 error_number = ZINT_ERROR_INVALID_OPTION;
+            } else if (error_number == ZINT_WARN_HRT_TRUNCATED) {
+                error_number = ZINT_ERROR_HRT_TRUNCATED;
+            } else { /* Shouldn't happen */
+                assert(0); /* Not reached */
+                error_number = ZINT_ERROR_ENCODING_PROBLEM;
             }
-            fmt = error_fmt;
         }
-        sprintf(error_buffer, fmt, error_string ? error_string : symbol->errtxt);
+        if (error_number >= ZINT_ERROR) {
+            sprintf(error_buffer, "Error %.93s", error_arg); /* Truncate if too long */
+        } else {
+            sprintf(error_buffer, "Warning %.91s", error_arg); /* Truncate if too long */
+        }
         strcpy(symbol->errtxt, error_buffer);
     }
 
@@ -249,14 +277,18 @@ INTERNAL int error_tag_test(struct zint_symbol *symbol, int error_number, const
 static int dump_plot(struct zint_symbol *symbol) {
     FILE *f;
     int i, r;
-    char hex[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8',
-        '9', 'A', 'B', 'C', 'D', 'E', 'F'};
+    static const char hex[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
     int space = 0;
+    const int output_to_stdout = symbol->output_options & BARCODE_STDOUT;
 
-    if (symbol->output_options & BARCODE_STDOUT) {
+    if (output_to_stdout) {
         f = stdout;
     } else {
+#ifdef _WIN32
+        f = out_win_fopen(symbol->outfile, "w");
+#else
         f = fopen(symbol->outfile, "w");
+#endif
         if (!f) {
             strcpy(symbol->errtxt, "201: Could not open output file");
             return ZINT_ERROR_FILE_ACCESS;
@@ -291,14 +323,28 @@ static int dump_plot(struct zint_symbol *symbol) {
             byt = byt << (4 - (symbol->width % 4));
             fputc(hex[byt], f);
         }
-        fputs("\n", f);
+        fputc('\n', f);
         space = 0;
     }
 
-    if (symbol->output_options & BARCODE_STDOUT) {
-        fflush(f);
+    if (ferror(f)) {
+        sprintf(symbol->errtxt, "795: Incomplete write to output (%d: %.30s)", errno, strerror(errno));
+        if (!output_to_stdout) {
+            (void) fclose(f);
+        }
+        return ZINT_ERROR_FILE_WRITE;
+    }
+
+    if (output_to_stdout) {
+        if (fflush(f) != 0) {
+            sprintf(symbol->errtxt, "796: Incomplete flush to output (%d: %.30s)", errno, strerror(errno));
+            return ZINT_ERROR_FILE_WRITE;
+        }
     } else {
-        fclose(f);
+        if (fclose(f) != 0) {
+            sprintf(symbol->errtxt, "792: Failure on closing output file (%d: %.30s)", errno, strerror(errno));
+            return ZINT_ERROR_FILE_WRITE;
+        }
     }
 
     return 0;
@@ -478,6 +524,7 @@ static int has_hrt(const int symbology) {
         case BARCODE_FIM:
         case BARCODE_PHARMA:
         case BARCODE_PHARMA_TWO:
+        case BARCODE_CEPNET:
         case BARCODE_PDF417:
         case BARCODE_PDF417COMP:
         case BARCODE_AUSPOST:
@@ -686,18 +733,16 @@ static int esc_base(struct zint_symbol *symbol, unsigned char *input_string, int
     return -1;
 }
 
-/* Helper to parse escape sequences */
-static int escape_char_process(struct zint_symbol *symbol, unsigned char *input_string, int *p_length) {
+/* Helper to parse escape sequences. If `escaped_string` NULL, calculates length only */
+static int escape_char_process(struct zint_symbol *symbol, unsigned char *input_string, int *p_length,
+            unsigned char *escaped_string) {
     const int length = *p_length;
-    int in_posn, out_posn;
+    int in_posn = 0, out_posn = 0;
     int ch;
     int val;
     int i;
     unsigned long unicode;
-    unsigned char *escaped_string = (unsigned char *) z_alloca(length + 1);
-
-    in_posn = 0;
-    out_posn = 0;
+    const int extra_escape_mode = (symbol->input_mode & EXTRA_ESCAPE_MODE) && symbol->symbology == BARCODE_CODE128;
 
     do {
         if (input_string[in_posn] == '\\') {
@@ -708,52 +753,87 @@ static int escape_char_process(struct zint_symbol *symbol, unsigned char *input_
             ch = input_string[in_posn + 1];
             /* NOTE: if add escape character, must also update regex in "frontend_qt/datawindow.php" */
             switch (ch) {
-                case '0': escaped_string[out_posn] = 0x00; /* Null */
+                case '0':
+                    if (escaped_string) escaped_string[out_posn] = 0x00; /* Null */
                     in_posn += 2;
                     break;
-                case 'E': escaped_string[out_posn] = 0x04; /* End of Transmission */
+                case '^': /* CODE128 specific */
+                    if (!extra_escape_mode) {
+                        strcpy(symbol->errtxt, "798: Escape '\\^' only valid for Code 128 in extra escape mode");
+                        return ZINT_ERROR_INVALID_DATA;
+                    }
+                    /* Pass thru unaltered */
+                    if (escaped_string) {
+                        escaped_string[out_posn++] = '\\';
+                        escaped_string[out_posn] = '^';
+                    } else {
+                        out_posn++;
+                    }
+                    in_posn += 2;
+                    if (in_posn < length) { /* Note allowing '\\^' on its own at end */
+                        if (escaped_string) {
+                            escaped_string[++out_posn] = input_string[in_posn++];
+                        } else {
+                            ++out_posn;
+                            in_posn++;
+                        }
+                    }
+                    break;
+                case 'E':
+                    if (escaped_string) escaped_string[out_posn] = 0x04; /* End of Transmission */
                     in_posn += 2;
                     break;
-                case 'a': escaped_string[out_posn] = 0x07; /* Bell */
+                case 'a':
+                    if (escaped_string) escaped_string[out_posn] = 0x07; /* Bell */
                     in_posn += 2;
                     break;
-                case 'b': escaped_string[out_posn] = 0x08; /* Backspace */
+                case 'b':
+                    if (escaped_string) escaped_string[out_posn] = 0x08; /* Backspace */
                     in_posn += 2;
                     break;
-                case 't': escaped_string[out_posn] = 0x09; /* Horizontal tab */
+                case 't':
+                    if (escaped_string) escaped_string[out_posn] = 0x09; /* Horizontal tab */
                     in_posn += 2;
                     break;
-                case 'n': escaped_string[out_posn] = 0x0a; /* Line feed */
+                case 'n':
+                    if (escaped_string) escaped_string[out_posn] = 0x0a; /* Line feed */
                     in_posn += 2;
                     break;
-                case 'v': escaped_string[out_posn] = 0x0b; /* Vertical tab */
+                case 'v':
+                    if (escaped_string) escaped_string[out_posn] = 0x0b; /* Vertical tab */
                     in_posn += 2;
                     break;
-                case 'f': escaped_string[out_posn] = 0x0c; /* Form feed */
+                case 'f':
+                    if (escaped_string) escaped_string[out_posn] = 0x0c; /* Form feed */
                     in_posn += 2;
                     break;
-                case 'r': escaped_string[out_posn] = 0x0d; /* Carriage return */
+                case 'r':
+                    if (escaped_string) escaped_string[out_posn] = 0x0d; /* Carriage return */
                     in_posn += 2;
                     break;
-                case 'e': escaped_string[out_posn] = 0x1b; /* Escape */
+                case 'e':
+                    if (escaped_string) escaped_string[out_posn] = 0x1b; /* Escape */
                     in_posn += 2;
                     break;
-                case 'G': escaped_string[out_posn] = 0x1d; /* Group Separator */
+                case 'G':
+                    if (escaped_string) escaped_string[out_posn] = 0x1d; /* Group Separator */
                     in_posn += 2;
                     break;
-                case 'R': escaped_string[out_posn] = 0x1e; /* Record Separator */
+                case 'R':
+                    if (escaped_string) escaped_string[out_posn] = 0x1e; /* Record Separator */
                     in_posn += 2;
                     break;
                 case 'd':
-                case 'o': /* Undocumented (as not very useful for most people) */
+                case 'o':
                 case 'x':
                     if ((val = esc_base(symbol, input_string, length, in_posn + 2, ch)) == -1) {
                         return ZINT_ERROR_INVALID_DATA;
                     }
-                    escaped_string[out_posn] = val;
+                    if (escaped_string) escaped_string[out_posn] = val;
                     in_posn += 4 + (ch != 'x');
                     break;
-                case '\\': escaped_string[out_posn] = '\\';
+                case '\\':
+                    if (escaped_string) escaped_string[out_posn] = '\\';
                     in_posn += 2;
                     break;
                 case 'u':
@@ -780,50 +860,63 @@ static int escape_char_process(struct zint_symbol *symbol, unsigned char *input_
                         sprintf(symbol->errtxt, "246: Invalid value for '\\%c' escape sequence in input data", ch);
                         return ZINT_ERROR_INVALID_DATA;
                     }
-                    if (unicode >= 0x800) {
-                        if (unicode >= 0x10000) {
-                            escaped_string[out_posn] = 0xf0 + (unsigned char) (unicode >> 16);
+                    if (unicode < 0x80) {
+                        if (escaped_string) escaped_string[out_posn] = (unsigned char) unicode;
+                    } else if (unicode < 0x800) {
+                        if (escaped_string) {
+                            escaped_string[out_posn++] = (unsigned char) (0xC0 | (unicode >> 6));
+                            escaped_string[out_posn] = (unsigned char) (0x80 | (unicode & 0x3F));
+                        } else {
                             out_posn++;
                         }
-                        escaped_string[out_posn] = 0xe0 + ((unicode & 0xf000) >> 12);
-                        out_posn++;
-                        escaped_string[out_posn] = 0x80 + ((unicode & 0x0fc0) >> 6);
-                        out_posn++;
-                        escaped_string[out_posn] = 0x80 + (unicode & 0x003f);
-                    } else if (unicode >= 0x80) {
-                        escaped_string[out_posn] = 0xc0 + ((unicode & 0x07c0) >> 6);
-                        out_posn++;
-                        escaped_string[out_posn] = 0x80 + (unicode & 0x003f);
+                    } else if (unicode < 0x10000) {
+                        if (escaped_string) {
+                            escaped_string[out_posn++] = (unsigned char) (0xE0 | (unicode >> 12));
+                            escaped_string[out_posn++] = (unsigned char) (0x80 | ((unicode >> 6) & 0x3F));
+                            escaped_string[out_posn] = (unsigned char) (0x80 | (unicode & 0x3F));
+                        } else {
+                            out_posn += 2;
+                        }
                     } else {
-                        escaped_string[out_posn] = unicode & 0x7f;
+                        if (escaped_string) {
+                            escaped_string[out_posn++] = (unsigned char) (0xF0 | (unicode >> 18));
+                            escaped_string[out_posn++] = (unsigned char) (0x80 | ((unicode >> 12) & 0x3F));
+                            escaped_string[out_posn++] = (unsigned char) (0x80 | ((unicode >> 6) & 0x3F));
+                            escaped_string[out_posn] = (unsigned char) (0x80 | (unicode & 0x3F));
+                        } else {
+                            out_posn += 3;
+                        }
                     }
                     in_posn += 6 + (ch == 'U') * 2;
                     break;
-                default: sprintf(symbol->errtxt, "234: Unrecognised escape character '\\%c' in input data", ch);
+                default:
+                    sprintf(symbol->errtxt, "234: Unrecognised escape character '\\%c' in input data", ch);
                     return ZINT_ERROR_INVALID_DATA;
                     break;
             }
         } else {
-            escaped_string[out_posn] = input_string[in_posn];
+            if (escaped_string) escaped_string[out_posn] = input_string[in_posn];
             in_posn++;
         }
         out_posn++;
     } while (in_posn < length);
 
-    memcpy(input_string, escaped_string, out_posn);
-    input_string[out_posn] = '\0';
+    if (escaped_string) {
+        escaped_string[out_posn] = '\0';
+    }
     *p_length = out_posn;
 
     return 0;
 }
 
-#ifdef ZINT_TEST /* Wrapper for direct testing */
-INTERNAL int escape_char_process_test(struct zint_symbol *symbol, unsigned char *input_string, int *length) {
-    return escape_char_process(symbol, input_string, length);
+#ifdef ZINT_TEST /* Wrapper for direct testing (also used by `testUtilZXingCPPCmp()` in "tests/testcommon.c") */
+INTERNAL int escape_char_process_test(struct zint_symbol *symbol, unsigned char *input_string, int *p_length,
+                unsigned char *escaped_string) {
+    return escape_char_process(symbol, input_string, p_length, escaped_string);
 }
 #endif
 
-/* Encode a barcode. If `length` is 0, `source` must be NUL-terminated. */
+/* Encode a barcode. If `length` is 0, `source` must be NUL-terminated */
 int ZBarcode_Encode(struct zint_symbol *symbol, const unsigned char *source, int length) {
     struct zint_seg segs[1];
 
@@ -845,7 +938,6 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
     unsigned char *local_source;
     struct zint_seg *local_segs;
     unsigned char *local_sources;
-    local_segs = (struct zint_seg *) z_alloca(sizeof(struct zint_seg) * (seg_count > 0 ? seg_count : 1));
 
     if (!symbol) return ZINT_ERROR_INVALID_DATA;
 
@@ -856,6 +948,7 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
     if (seg_count > ZINT_MAX_SEG_COUNT) {
         return error_tag(symbol, ZINT_ERROR_INVALID_DATA, "771: Too many input segments (max 256)");
     }
+    local_segs = (struct zint_seg *) z_alloca(sizeof(struct zint_seg) * (seg_count > 0 ? seg_count : 1));
 
     if ((symbol->input_mode & 0x07) > 2) {
         symbol->input_mode = DATA_MODE; /* Reset completely TODO: in future, warn/error */
@@ -886,10 +979,22 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
             }
             return error_tag(symbol, ZINT_ERROR_INVALID_DATA, NULL);
         }
-        if (local_segs[i].length > ZINT_MAX_DATA_LEN) {
-            return error_tag(symbol, ZINT_ERROR_TOO_LONG, "777: Input data too long");
+        if (symbol->input_mode & ESCAPE_MODE) { /* Calculate de-escaped length for check against ZINT_MAX_DATA_LEN */
+            int escaped_len = local_segs[i].length;
+            error_number = escape_char_process(symbol, local_segs[i].source, &escaped_len, NULL /*escaped_string*/);
+            if (error_number != 0) { /* Only returns errors, not warnings */
+                return error_tag(symbol, error_number, NULL);
+            }
+            if (escaped_len > ZINT_MAX_DATA_LEN) {
+                return error_tag(symbol, ZINT_ERROR_TOO_LONG, "797: Input data too long");
+            }
+            total_len += escaped_len;
+        } else {
+            if (local_segs[i].length > ZINT_MAX_DATA_LEN) {
+                return error_tag(symbol, ZINT_ERROR_TOO_LONG, "777: Input data too long");
+            }
+            total_len += local_segs[i].length;
         }
-        total_len += local_segs[i].length;
     }
 
     if (total_len == 0) {
@@ -900,14 +1005,19 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
         const int len = local_segs[0].length;
         const int primary_len = symbol->primary[0] ? (int) strlen(symbol->primary) : 0;
         char name[32];
+        char source[151], primary[151]; /* 30*5 + 1 = 151 */
         (void) ZBarcode_BarcodeName(symbol->symbology, name);
-        printf("\nZBarcode_Encode_Segs: %s (%d), input_mode: 0x%X, ECI: %d, option_1: %d, option_2: %d"
-                ", option_3: %d,\n                      scale: %g, output_options: 0x%X, fg: %s, bg: %s"
-                ", seg_count: %d,\n                      %ssource%s (%d): \"%.*s\", %sprimary (%d): \"%.20s\"\n",
+        debug_print_escape(local_segs[0].source, len > 30 ? 30 : len, source);
+        debug_print_escape((const unsigned char *) symbol->primary, primary_len > 30 ? 30 : primary_len, primary);
+        printf("\nZBarcode_Encode_Segs: %s (%d), input_mode: 0x%X, ECI: %d, option_1/2/3: (%d, %d, %d)\n"
+                "                      scale: %g, output_options: 0x%X, fg: %s, bg: %s, seg_count: %d,\n"
+                "                      %ssource%s (%d): \"%s\",\n"
+                "                      %sprimary (%d): \"%s\"\n",
                 name, symbol->symbology, symbol->input_mode, symbol->eci, symbol->option_1, symbol->option_2,
                 symbol->option_3, symbol->scale, symbol->output_options, symbol->fgcolour, symbol->bgcolour,
-                seg_count, len > 20 ? "First 20 " : "", seg_count > 1 ? "[0]" : "", len, len > 20 ? 20 : len,
-                local_segs[0].source, primary_len > 20 ? "First 20 " : "", primary_len, symbol->primary);
+                seg_count, len > 30 ? "first 30 " : "", seg_count > 1 ? "[0]" : "", len, source,
+                primary_len > 30 ? "first 30 " : "", primary_len, primary);
+        fflush(stdout);
     }
 
     if (total_len > ZINT_MAX_DATA_LEN) {
@@ -1067,6 +1177,9 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
     if ((symbol->guard_descent < 0.0f) || (symbol->guard_descent > 50.0f)) {
         return error_tag(symbol, ZINT_ERROR_INVALID_OPTION, "769: Guard bar descent out of range (0 to 50)");
     }
+    if ((symbol->text_gap < -5.0f) || (symbol->text_gap > 10.0f)) {
+        return error_tag(symbol, ZINT_ERROR_INVALID_OPTION, "219: Text gap out of range (-5 to 10)");
+    }
     if ((symbol->whitespace_width < 0) || (symbol->whitespace_width > 100)) {
         return error_tag(symbol, ZINT_ERROR_INVALID_OPTION, "766: Whitespace width out of range (0 to 100)");
     }
@@ -1080,6 +1193,9 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
     if (symbol->rows >= 200) { /* Check for stacking too many symbols */
         return error_tag(symbol, ZINT_ERROR_TOO_LONG, "770: Too many stacked symbols");
     }
+    if (symbol->rows < 0) { /* Silently defend against out-of-bounds access */
+        symbol->rows = 0;
+    }
 
     if ((symbol->input_mode & 0x07) == GS1_MODE && !gs1_compliant(symbol->symbology)) {
         return error_tag(symbol, ZINT_ERROR_INVALID_OPTION, "220: Selected symbology does not support GS1 mode");
@@ -1101,27 +1217,30 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
 
     local_sources = (unsigned char *) z_alloca(total_len + seg_count);
 
+    /* Copy input, de-escaping if required */
     for (i = 0, local_source = local_sources; i < seg_count; i++) {
         local_segs[i].source = local_source;
-        memcpy(local_segs[i].source, segs[i].source, local_segs[i].length);
-        local_segs[i].source[local_segs[i].length] = '\0';
+        if (symbol->input_mode & ESCAPE_MODE) {
+            /* Checked already */
+            (void) escape_char_process(symbol, segs[i].source, &local_segs[i].length, local_segs[i].source);
+        } else {
+            memcpy(local_segs[i].source, segs[i].source, local_segs[i].length);
+            local_segs[i].source[local_segs[i].length] = '\0';
+        }
         local_source += local_segs[i].length + 1;
     }
 
-    /* Start acting on input mode */
-    if (symbol->input_mode & ESCAPE_MODE) {
-        for (i = 0; i < seg_count; i++) {
-            error_number = escape_char_process(symbol, local_segs[i].source, &local_segs[i].length);
-            if (error_number != 0) { /* Only returns errors, not warnings */
-                return error_tag(symbol, error_number, NULL);
-            }
+    if ((symbol->input_mode & ESCAPE_MODE) && symbol->primary[0] && strchr(symbol->primary, '\\') != NULL) {
+        char primary[sizeof(symbol->primary)];
+        int primary_len = (int) strlen(symbol->primary);
+        if (primary_len >= (int) sizeof(symbol->primary)) {
+            return error_tag(symbol, ZINT_ERROR_INVALID_DATA, "799: Invalid primary string");
         }
-        if (symbol->primary[0]) {
-            int primary_len = (int) strlen(symbol->primary);
-            error_number = escape_char_process(symbol, (unsigned char *) symbol->primary, &primary_len);
-            if (error_number != 0) { /* Only returns errors, not warnings */
-                return error_tag(symbol, error_number, NULL);
-            }
+        ustrcpy(primary, symbol->primary);
+        error_number = escape_char_process(symbol, (unsigned char *) primary, &primary_len,
+                                            (unsigned char *) symbol->primary);
+        if (error_number != 0) { /* Only returns errors, not warnings */
+            return error_tag(symbol, error_number, NULL);
         }
     }
 
@@ -1153,7 +1272,7 @@ int ZBarcode_Encode_Segs(struct zint_symbol *symbol, const struct zint_seg segs[
                 local_segs[0].length = (int) ustrlen(reduced);
             }
         } else {
-            return error_tag(symbol, ZINT_ERROR_INVALID_OPTION, "220: Selected symbology does not support GS1 mode");
+            return error_tag(symbol, ZINT_ERROR_INVALID_OPTION, "210: Selected symbology does not support GS1 mode");
         }
     }
 
@@ -1413,11 +1532,15 @@ int ZBarcode_Encode_File(struct zint_symbol *symbol, const char *filename) {
         return error_tag(symbol, ZINT_ERROR_INVALID_DATA, "239: Filename NULL");
     }
 
-    if (!strcmp(filename, "-")) {
+    if (strcmp(filename, "-") == 0) {
         file = stdin;
         fileLen = ZINT_MAX_DATA_LEN;
     } else {
+#ifdef _WIN32
+        file = out_win_fopen(filename, "rb");
+#else
         file = fopen(filename, "rb");
+#endif
         if (!file) {
             sprintf(symbol->errtxt, "229: Unable to read input file (%d: %.30s)", errno, strerror(errno));
             return error_tag(symbol, ZINT_ERROR_INVALID_DATA, NULL);
@@ -1425,26 +1548,36 @@ int ZBarcode_Encode_File(struct zint_symbol *symbol, const char *filename) {
         file_opened = 1;
 
         /* Get file length */
-        fseek(file, 0, SEEK_END);
+        if (fseek(file, 0, SEEK_END) != 0) {
+            sprintf(symbol->errtxt, "797: Unable to seek input file (%d: %.30s)", errno, strerror(errno));
+            (void) fclose(file);
+            return error_tag(symbol, ZINT_ERROR_INVALID_DATA, NULL);
+        }
+
         fileLen = ftell(file);
-        fseek(file, 0, SEEK_SET);
 
         /* On many Linux distros ftell() returns LONG_MAX not -1 on error */
         if (fileLen <= 0 || fileLen == LONG_MAX) {
-            fclose(file);
+            (void) fclose(file);
             return error_tag(symbol, ZINT_ERROR_INVALID_DATA, "235: Input file empty or unseekable");
         }
         if (fileLen > ZINT_MAX_DATA_LEN) {
-            fclose(file);
+            (void) fclose(file);
             return error_tag(symbol, ZINT_ERROR_TOO_LONG, "230: Input file too long");
         }
+
+        if (fseek(file, 0, SEEK_SET) != 0) {
+            sprintf(symbol->errtxt, "793: Unable to seek input file (%d: %.30s)", errno, strerror(errno));
+            (void) fclose(file);
+            return error_tag(symbol, ZINT_ERROR_INVALID_DATA, NULL);
+        }
     }
 
     /* Allocate memory */
     buffer = (unsigned char *) malloc(fileLen);
     if (!buffer) {
         if (file_opened) {
-            fclose(file);
+            (void) fclose(file);
         }
         return error_tag(symbol, ZINT_ERROR_MEMORY, "231: Insufficient memory for file read buffer");
     }
@@ -1455,17 +1588,21 @@ int ZBarcode_Encode_File(struct zint_symbol *symbol, const char *filename) {
         n = fread(buffer + nRead, 1, fileLen - nRead, file);
         if (ferror(file)) {
             sprintf(symbol->errtxt, "241: Input file read error (%d: %.30s)", errno, strerror(errno));
-            if (file_opened) {
-                fclose(file);
-            }
             free(buffer);
+            if (file_opened) {
+                (void) fclose(file);
+            }
             return error_tag(symbol, ZINT_ERROR_INVALID_DATA, NULL);
         }
         nRead += n;
     } while (!feof(file) && (0 < n) && ((long) nRead < fileLen));
 
     if (file_opened) {
-        fclose(file);
+        if (fclose(file) != 0) {
+            sprintf(symbol->errtxt, "794: Failure on closing input file (%d: %.30s)", errno, strerror(errno));
+            free(buffer);
+            return error_tag(symbol, ZINT_ERROR_INVALID_DATA, NULL);
+        }
     }
     ret = ZBarcode_Encode(symbol, buffer, (int) nRead);
     free(buffer);
@@ -1729,8 +1866,8 @@ unsigned int ZBarcode_Cap(int symbol_id, unsigned int cap_flag) {
     if ((cap_flag & ZINT_CAP_STACKABLE) && is_stackable(symbol_id)) {
         result |= ZINT_CAP_STACKABLE;
     }
-    if ((cap_flag & ZINT_CAP_EXTENDABLE) && is_extendable(symbol_id)) {
-        result |= ZINT_CAP_EXTENDABLE;
+    if ((cap_flag & ZINT_CAP_EANUPC) && is_upcean(symbol_id)) {
+        result |= ZINT_CAP_EANUPC;
     }
     if ((cap_flag & ZINT_CAP_COMPOSITE) && is_composite(symbol_id)) {
         result |= ZINT_CAP_COMPOSITE;
