@@ -1,7 +1,7 @@
 /* postal.c - Handles POSTNET, PLANET, CEPNet, FIM. RM4SCC and Flattermarken */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2008-2022 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2008-2023 Robin Stuart <rstuart114@gmail.com>
     Including bug fixes by Bryan Hatton
 
     Redistribution and use in source and binary forms, with or without
@@ -559,12 +559,12 @@ INTERNAL int kix(struct zint_symbol *symbol, unsigned char source[], int length)
 
 /* Handles DAFT Code symbols */
 INTERNAL int daft(struct zint_symbol *symbol, unsigned char source[], int length) {
-    int posns[100];
+    int posns[576];
     int loopey;
     int writer;
 
-    if (length > 100) {
-        strcpy(symbol->errtxt, "492: Input too long (100 character maximum)");
+    if (length > 576) { /* 576 * 2 = 1152 */
+        strcpy(symbol->errtxt, "492: Input too long (576 character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
     to_upper(source, length);
@@ -593,7 +593,7 @@ INTERNAL int daft(struct zint_symbol *symbol, unsigned char source[], int length
             symbol->height = 8.0f;
         }
         symbol->row_height[1] = stripf(symbol->height * t_ratio);
-        symbol->row_height[0] = stripf((symbol->height - symbol->row_height[1]) / 2.0);
+        symbol->row_height[0] = stripf((symbol->height - symbol->row_height[1]) / 2.0f);
     } else {
         symbol->row_height[0] = 3.0f;
         symbol->row_height[1] = 2.0f;
@@ -610,11 +610,11 @@ INTERNAL int daft(struct zint_symbol *symbol, unsigned char source[], int length
 /* Flattermarken - Not really a barcode symbology! */
 INTERNAL int flat(struct zint_symbol *symbol, unsigned char source[], int length) {
     int loop, error_number = 0;
-    char dest[512]; /* 90 * 4 + 1 ~ */
+    char dest[512]; /* 128 * 4 = 512 */
     char *d = dest;
 
-    if (length > 90) {
-        strcpy(symbol->errtxt, "494: Input too long (90 character maximum)");
+    if (length > 128) { /* 128 * 9 = 1152 */
+        strcpy(symbol->errtxt, "494: Input too long (128 character maximum)");
         return ZINT_ERROR_TOO_LONG;
     }
     if (!is_sane(NEON_F, source, length)) {
