@@ -14,10 +14,13 @@ function run_zxingcpp_test() {
 }
 
 run_zxingcpp_test "test_2of5" "encode"
+run_zxingcpp_test "test_aztec" "large"
 run_zxingcpp_test "test_aztec" "encode"
 run_zxingcpp_test "test_aztec" "encode_segs"
+run_zxingcpp_test "test_aztec" "fuzz"
 run_zxingcpp_test "test_codablock" "input"
 run_zxingcpp_test "test_codablock" "encode"
+run_zxingcpp_test "test_codablock" "fuzz"
 run_zxingcpp_test "test_code" "encode"
 run_zxingcpp_test "test_code128" "input"
 run_zxingcpp_test "test_code128" "encode"
@@ -29,9 +32,7 @@ run_zxingcpp_test "test_dmatrix" "encode_segs"
 run_zxingcpp_test "test_dotcode" "input"
 run_zxingcpp_test "test_dotcode" "encode"
 run_zxingcpp_test "test_dotcode" "encode_segs"
-run_zxingcpp_test "test_hanxin" "input"
-run_zxingcpp_test "test_hanxin" "encode"
-run_zxingcpp_test "test_hanxin" "encode_segs"
+run_zxingcpp_test "test_hanxin"
 run_zxingcpp_test "test_mailmark" "2d_encode"
 run_zxingcpp_test "test_maxicode" "input"
 run_zxingcpp_test "test_maxicode" "encode"
@@ -41,6 +42,7 @@ run_zxingcpp_test "test_pdf417" "reader_init"
 run_zxingcpp_test "test_pdf417" "input"
 run_zxingcpp_test "test_pdf417" "encode"
 run_zxingcpp_test "test_pdf417" "encode_segs"
+run_zxingcpp_test "test_pdf417" "fuzz"
 run_zxingcpp_test "test_qr"
 run_zxingcpp_test "test_rss" "binary_div_modulo_divisor"
 run_zxingcpp_test "test_rss" "examples"
