@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2008 by BogDan Vatra                                    *
  *   bogdan@licentia.eu                                                    *
- *   Copyright (C) 2010-2022 Robin Stuart                                  *
+ *   Copyright (C) 2010-2023 Robin Stuart                                  *
  *                                                                         *
  *   This program is free software: you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -26,21 +26,86 @@
 #include "qzint.h"
 #include <math.h>
 #include <stdio.h>
+#include <QFontDatabase>
 #include <QFontMetrics>
 /* The following include is necessary to compile with Qt 5.15 on Windows; Qt 5.7 did not require it */
 #include <QPainterPath>
+#include <QRegularExpression>
+#include "../backend/fonts/normal_ttf.h" /* Arimo */
+#include "../backend/fonts/upcean_ttf.h" /* OCR-B subset (digits, "<", ">") */
 
 // Shorthand
 #define QSL QStringLiteral
 
 namespace Zint {
-    static const char fontStyle[] = "Helvetica";
-    static const char fontStyleError[] = "Helvetica";
-    static const int fontSizeError = 14; /* Point size */
-
     static const int maxSegs = 256;
     static const int maxCLISegs = 10; /* CLI restricted to 10 segments (including main data) */
 
+    /* Matches RGB(A) hex string or CMYK decimal "C,M,Y,K" percentage string */
+    static const QRegularExpression colorRE(
+                                QSL("^([0-9A-Fa-f]{6}([0-9A-Fa-f]{2})?)|(((100|[0-9]{0,2}),){3}(100|[0-9]{0,2}))$"));
+
+    static const QString normalFontFamily = QSL("Arimo"); /* Sans-serif metrically compatible with Arial */
+    static const QString upceanFontFamily = QSL("OCRB"); /* Monospace OCR-B */
+    static const QString fontFamilyError = QSL("Arimo");
+    static const int fontSizeError = 14; /* Point size */
+
+    static int normalFontID = -2; /* Use -2 as `addApplicationFontFromData()` returns -1 on error */
+
+    /* Load Arimo from static array */
+    static int loadNormalFont() {
+        static const QByteArray normalFontArray
+                                            = QByteArray::fromRawData((const char *) normal_ttf, sizeof(normal_ttf));
+
+        normalFontID = QFontDatabase::addApplicationFontFromData(normalFontArray);
+        return normalFontID;
+    }
+
+    static int upceanFontID = -2; /* Use -2 as `addApplicationFontFromData()` returns -1 on error */
+
+    /* Load OCR-B EAN/UPC subset from static array */
+    static int loadUpceanFont() {
+        static const QByteArray upceanFontArray
+                                            = QByteArray::fromRawData((const char *) upcean_ttf, sizeof(upcean_ttf));
+
+        upceanFontID = QFontDatabase::addApplicationFontFromData(upceanFontArray);
+        return upceanFontID;
+    }
+
+    /* Helper to convert QColor to RGB(A) hex string */
+    static QString qcolor_to_str(const QColor &color) {
+        if (color.alpha() == 0xFF) {
+            return QString::asprintf("%02X%02X%02X", color.red(), color.green(), color.blue());
+        }
+        return QString::asprintf("%02X%02X%02X%02X", color.red(), color.green(), color.blue(), color.alpha());
+    }
+
+    /* Helper to convert RGB(A) hex string or CMYK decimal "C,M,Y,K" percentage string) to QColor */
+    static QColor str_to_qcolor(const QString &text) {
+        QColor color;
+        int r, g, b, a;
+        if (text.contains(',')) {
+            int comma1 = text.indexOf(',');
+            int comma2 = text.indexOf(',', comma1 + 1);
+            int comma3 = text.indexOf(',', comma2 + 1);
+            int black = 100 - text.mid(comma3 + 1).toInt();
+            int val = 100 - text.mid(0, comma1).toInt();
+            r = (int) roundf((0xFF * val * black) / 10000.0f);
+            val = 100 - text.mid(comma1 + 1, comma2 - comma1 - 1).toInt();
+            g = (int) roundf((0xFF * val * black) / 10000.0f);
+            val = 100 - text.mid(comma2 + 1, comma3 - comma2 - 1).toInt();
+            b = (int) roundf((0xFF * val * black) / 10000.0f);
+            a = 0xFF;
+        } else {
+            r = text.mid(0, 2).toInt(nullptr, 16);
+            g = text.mid(2, 2).toInt(nullptr, 16);
+            b = text.mid(4, 2).toInt(nullptr, 16);
+            a = text.length() == 8 ? text.mid(6, 2).toInt(nullptr, 16) : 0xFF;
+        }
+        color.setRgb(r, g, b, a);
+        return color;
+    }
+
     /* Helper to convert ECI combo index to ECI value */
     static int ECIIndexToECI(const int ECIIndex) {
         int ret;
@@ -60,7 +125,7 @@ namespace Zint {
         return ret;
     }
 
-    /* Helper to calculate max right and bottom of elements for fudging render() */
+    /* Helper to calculate max right and bottom of elements for fudging `render()` */
     static void getMaxRectsRightBottom(struct zint_vector *vector, int &maxRight, int &maxBottom) {
         struct zint_vector_rect *rect;
         struct zint_vector_hexagon *hex;
@@ -94,6 +159,7 @@ namespace Zint {
                 maxBottom = circle->y + circle->diameter + circle->width;
             }
         }
+
         // TODO: Strings?
     }
 
@@ -109,7 +175,8 @@ namespace Zint {
             m_scale(1.0f),
             m_dotty(false), m_dot_size(4.0f / 5.0f),
             m_guardDescent(5.0f),
-            m_fgColor(Qt::black), m_bgColor(Qt::white), m_cmyk(false),
+            m_textGap(1.0f),
+            m_fgStr(QSL("000000")), m_bgStr(QSL("FFFFFF")), m_cmyk(false),
             m_borderType(0), m_borderWidth(0),
             m_whitespace(0), m_vwhitespace(0),
             m_fontSetting(0),
@@ -121,6 +188,8 @@ namespace Zint {
             m_eci(0),
             m_gs1parens(false), m_gs1nocheck(false),
             m_reader_init(false),
+            m_guard_whitespace(false),
+            m_embed_vector_font(false),
             m_warn_level(WARN_DEFAULT), m_debug(false),
             m_encodedWidth(0), m_encodedRows(0), m_encodedHeight(0.0f),
             m_vectorWidth(0.0f), m_vectorHeight(0.0f),
@@ -140,7 +209,7 @@ namespace Zint {
         m_lastError.clear();
 
         if (m_zintSymbol) {
-            ZBarcode_Clear(m_zintSymbol);
+            ZBarcode_Reset(m_zintSymbol);
         } else if (!(m_zintSymbol = ZBarcode_Create())) {
             m_error = ZINT_ERROR_MEMORY;
             m_lastError = QSL("Insufficient memory for Zint structure");
@@ -175,14 +244,14 @@ namespace Zint {
         if (m_reader_init) {
             m_zintSymbol->output_options |= READER_INIT;
         }
-        strcpy(m_zintSymbol->fgcolour, m_fgColor.name().toLatin1().right(6));
-        if (m_fgColor.alpha() != 0xFF) {
-            strcat(m_zintSymbol->fgcolour, m_fgColor.name(QColor::HexArgb).toLatin1().mid(1,2));
+        if (m_guard_whitespace) {
+            m_zintSymbol->output_options |= EANUPC_GUARD_WHITESPACE;
         }
-        strcpy(m_zintSymbol->bgcolour, m_bgColor.name().toLatin1().right(6));
-        if (m_bgColor.alpha() != 0xFF) {
-            strcat(m_zintSymbol->bgcolour, m_bgColor.name(QColor::HexArgb).toLatin1().mid(1,2));
+        if (m_embed_vector_font) {
+            m_zintSymbol->output_options |= EMBED_VECTOR_FONT;
         }
+        strcpy(m_zintSymbol->fgcolour, m_fgStr.toLatin1().left(15));
+        strcpy(m_zintSymbol->bgcolour, m_bgStr.toLatin1().left(15));
         strcpy(m_zintSymbol->primary, m_primaryMessage.toLatin1().left(127));
         m_zintSymbol->option_1 = m_option_1;
         m_zintSymbol->option_2 = m_option_2;
@@ -199,6 +268,7 @@ namespace Zint {
         m_zintSymbol->dpmm = m_dpmm;
         m_zintSymbol->dot_size = m_dot_size;
         m_zintSymbol->guard_descent = m_guardDescent;
+        m_zintSymbol->text_gap = m_textGap;
         m_zintSymbol->structapp = m_structapp;
         m_zintSymbol->warn_level = m_warn_level;
         m_zintSymbol->debug = m_debug ? ZINT_DEBUG_PRINT : 0;
@@ -412,22 +482,48 @@ namespace Zint {
         memset(&m_structapp, 0, sizeof(m_structapp));
     }
 
-    /* Foreground colour */
+    /* Foreground colour (may be RGB(A) hex string or CMYK decimal "C,M,Y,K" percentage string) */
+    QString QZint::fgStr() const {
+        return m_fgStr;
+    }
+
+    bool QZint::setFgStr(const QString& fgStr) {
+        if (fgStr.indexOf(colorRE) == 0) {
+            m_fgStr = fgStr;
+            return true;
+        }
+        return false;
+    }
+
+    /* Foreground colour as QColor */
     QColor QZint::fgColor() const {
-        return m_fgColor;
+        return str_to_qcolor(m_fgStr);
     }
 
     void QZint::setFgColor(const QColor& fgColor) {
-        m_fgColor = fgColor;
+        m_fgStr = qcolor_to_str(fgColor);
     }
 
-    /* Background colour */
+    /* Background colour (may be RGB(A) hex string or CMYK decimal "C,M,Y,K" percentage string) */
+    QString QZint::bgStr() const {
+        return m_bgStr;
+    }
+
+    bool QZint::setBgStr(const QString& bgStr) {
+        if (bgStr.indexOf(colorRE) == 0) {
+            m_bgStr = bgStr;
+            return true;
+        }
+        return false;
+    }
+
+    /* Background colour as QColor */
     QColor QZint::bgColor() const {
-        return m_bgColor;
+        return str_to_qcolor(m_bgStr);
     }
 
     void QZint::setBgColor(const QColor& bgColor) {
-        m_bgColor = bgColor;
+        m_bgStr = qcolor_to_str(bgColor);
     }
 
     /* Use CMYK colour space (Encapsulated PostScript and TIF) */
@@ -510,7 +606,16 @@ namespace Zint {
         }
     }
 
-    /* Show (true) or hide (false) Human Readable Text */
+    /* Text gap */
+    float QZint::textGap() const {
+        return m_textGap;
+    }
+
+    void QZint::setTextGap(float textGap) {
+        m_textGap = textGap;
+    }
+
+    /* Show (true) or hide (false) Human Readable Text (HRT) */
     bool QZint::showText() const {
         return m_show_hrt;
     }
@@ -629,6 +734,24 @@ namespace Zint {
         m_reader_init = readerInit;
     }
 
+    /* Whether to add quiet zone indicators ("<", ">") to HRT (EAN/UPC) */
+    bool QZint::guardWhitespace() const {
+        return m_guard_whitespace;
+    }
+
+    void QZint::setGuardWhitespace(bool guardWhitespace) {
+        m_guard_whitespace = guardWhitespace;
+    }
+
+    /* Whether to embed the font in vector output - currently only for SVG output of EAN/UPC */
+    bool QZint::embedVectorFont() const {
+        return m_embed_vector_font;
+    }
+
+    void QZint::setEmbedVectorFont(bool embedVectorFont) {
+        m_embed_vector_font = embedVectorFont;
+    }
+
     /* Affects error/warning value returned by Zint API (see `getError()` below) */
     int QZint::warnLevel() const {
         return m_warn_level;
@@ -694,8 +817,12 @@ namespace Zint {
         return ZBarcode_Cap(symbology ? symbology : m_symbol, ZINT_CAP_STACKABLE);
     }
 
-    bool QZint::isExtendable(int symbology) const {
-        return ZBarcode_Cap(symbology ? symbology : m_symbol, ZINT_CAP_EXTENDABLE);
+    bool QZint::isEANUPC(int symbology) const {
+        return ZBarcode_Cap(symbology ? symbology : m_symbol, ZINT_CAP_EANUPC);
+    }
+
+    bool QZint::isExtendable(int symbology) const { /* Legacy - same as `isEANUPC()` */
+        return ZBarcode_Cap(symbology ? symbology : m_symbol, ZINT_CAP_EANUPC);
     }
 
     bool QZint::isComposite(int symbology) const {
@@ -776,7 +903,7 @@ namespace Zint {
 
     bool QZint::save_to_file(const QString& filename) {
         if (resetSymbol()) {
-            strcpy(m_zintSymbol->outfile, filename.toLatin1().left(255));
+            strcpy(m_zintSymbol->outfile, filename.toUtf8().left(255));
             if (m_segs.empty()) {
                 QByteArray bstr = m_text.toUtf8();
                 m_error = ZBarcode_Encode_and_Print(m_zintSymbol, (unsigned char *) bstr.data(), bstr.length(),
@@ -848,14 +975,15 @@ namespace Zint {
         struct zint_vector_hexagon *hex;
         struct zint_vector_circle *circle;
         struct zint_vector_string *string;
-
+        QColor fgColor = str_to_qcolor(m_fgStr);
+        QColor bgColor = str_to_qcolor(m_bgStr);
         encode();
 
         painter.save();
 
         if (m_error >= ZINT_ERROR) {
             painter.setRenderHint(QPainter::Antialiasing);
-            QFont font(fontStyleError, fontSizeError);
+            QFont font(fontFamilyError, fontSizeError);
             painter.setFont(font);
             painter.drawText(paintRect, Qt::AlignCenter | Qt::TextWordWrap, m_lastError);
             painter.restore();
@@ -897,8 +1025,10 @@ namespace Zint {
         painter.translate(xtr, ytr);
         painter.scale(scale, scale);
 
-        QBrush bgBrush(m_bgColor);
-        painter.fillRect(QRectF(0, 0, gwidth, gheight), bgBrush);
+        QBrush bgBrush(bgColor);
+        if (bgColor.alpha() != 0) {
+            painter.fillRect(QRectF(0, 0, gwidth, gheight), bgBrush);
+        }
 
         // Plot rectangles
         rect = m_zintSymbol->vector->rectangles;
@@ -908,7 +1038,7 @@ namespace Zint {
             QBrush brush(Qt::SolidPattern);
             while (rect) {
                 if (rect->colour == -1) {
-                    brush.setColor(m_fgColor);
+                    brush.setColor(fgColor);
                 } else {
                     brush.setColor(colourToQtColor(rect->colour));
                 }
@@ -924,7 +1054,7 @@ namespace Zint {
         hex = m_zintSymbol->vector->hexagons;
         if (hex) {
             painter.setRenderHint(QPainter::Antialiasing);
-            QBrush fgBrush(m_fgColor);
+            QBrush fgBrush(fgColor);
             qreal previous_diameter = 0.0, radius = 0.0, half_radius = 0.0, half_sqrt3_radius = 0.0;
             while (hex) {
                 if (previous_diameter != hex->diameter) {
@@ -953,20 +1083,20 @@ namespace Zint {
         if (circle) {
             painter.setRenderHint(QPainter::Antialiasing);
             QPen p;
-            QBrush fgBrush(m_fgColor);
+            QBrush fgBrush(fgColor);
             qreal previous_diameter = 0.0, radius = 0.0;
             while (circle) {
                 if (previous_diameter != circle->diameter) {
                     previous_diameter = circle->diameter;
                     radius = 0.5 * previous_diameter;
                 }
-                if (circle->colour) { // Set means use background colour
-                    p.setColor(m_bgColor);
+                if (circle->colour) { // Set means use background colour (legacy, no longer used)
+                    p.setColor(bgColor);
                     p.setWidthF(circle->width);
                     painter.setPen(p);
                     painter.setBrush(circle->width ? Qt::NoBrush : bgBrush);
                 } else {
-                    p.setColor(m_fgColor);
+                    p.setColor(fgColor);
                     p.setWidthF(circle->width);
                     painter.setPen(p);
                     painter.setBrush(circle->width ? Qt::NoBrush : fgBrush);
@@ -979,13 +1109,19 @@ namespace Zint {
         // Plot text
         string = m_zintSymbol->vector->strings;
         if (string) {
+            if (normalFontID == -2) { /* First time? */
+                loadNormalFont();
+            }
+            if (upceanFontID == -2) { /* First time? */
+                loadUpceanFont();
+            }
             painter.setRenderHint(QPainter::Antialiasing);
             QPen p;
-            p.setColor(m_fgColor);
+            p.setColor(fgColor);
             painter.setPen(p);
-            bool bold = (m_zintSymbol->output_options & BOLD_TEXT)
-                            && (!isExtendable() || (m_zintSymbol->output_options & SMALL_TEXT));
-            QFont font(fontStyle, -1 /*pointSize*/, bold ? QFont::Bold : -1);
+            bool bold = (m_zintSymbol->output_options & BOLD_TEXT) && !isEANUPC();
+            QFont font(isEANUPC() ? upceanFontFamily : normalFontFamily, -1 /*pointSize*/,
+                        bold ? QFont::Bold : -1);
             while (string) {
                 font.setPixelSize(string->fsize);
                 painter.setFont(font);
@@ -995,7 +1131,7 @@ namespace Zint {
                     painter.drawText(QPointF(string->x, string->y), content);
                 } else {
                     QFontMetrics fm(painter.fontMetrics());
-                    int width = fm.boundingRect(content).width();
+                    int width = fm.horizontalAdvance(content);
                     if (string->halign == 2) { /* Right align */
                         painter.drawText(QPointF(string->x - width, string->y), content);
                     } else { /* Centre align */
@@ -1080,6 +1216,8 @@ namespace Zint {
                     const bool autoHeight, const float heightPerRow, const QString& outfile,
                     const QZintXdimDpVars *xdimdpVars) const {
         QString cmd(win && !noEXE ? QSL("zint.exe") : QSL("zint"));
+        const bool nobackground = bgColor().alpha() == 0;
+        const bool notext = hasHRT() && !showText();
 
         char name_buf[32];
         if (barcodeNames && ZBarcode_BarcodeName(m_symbol, name_buf) == 0) {
@@ -1089,12 +1227,12 @@ namespace Zint {
             arg_int(cmd, longOptOnly ? "--barcode=" : "-b ", m_symbol);
         }
 
-        if (isExtendable()) {
+        if (isEANUPC()) {
             arg_int(cmd, "--addongap=", option2());
         }
 
-        if (bgColor() != Qt::white && bgColor() != QColor(0xFF, 0xFF, 0xFF, 0)) {
-            arg_color(cmd, "--bg=", bgColor());
+        if (bgStr() != QSL("FFFFFF") && !nobackground) {
+            arg_str(cmd, "--bg=", bgStr());
         }
 
         bool default_bind = false, default_bind_top = false, default_box = false, default_border = false;
@@ -1120,7 +1258,7 @@ namespace Zint {
         if (!default_bind_top) {
             arg_bool(cmd, "--bindtop", borderType() & BARCODE_BIND_TOP);
         }
-        arg_bool(cmd, "--bold", fontSetting() & BOLD_TEXT);
+        arg_bool(cmd, "--bold", !notext && (fontSetting() & BOLD_TEXT) && !isEANUPC());
         if (!default_border) {
             arg_int(cmd, "--border=", borderWidth());
         }
@@ -1152,7 +1290,8 @@ namespace Zint {
         }
 
         if (m_symbol == BARCODE_DATAMATRIX || m_symbol == BARCODE_HIBC_DM) {
-            arg_bool(cmd, "--dmre", option3() == DM_DMRE);
+            arg_bool(cmd, "--dmiso144", (option3() & DM_ISO_144) == DM_ISO_144);
+            arg_bool(cmd, "--dmre", (option3() & 0x7F) == DM_DMRE);
         }
 
         if ((m_symbol == BARCODE_DOTCODE || (isDotty() && dotty())) && dotSize() != 0.8f) {
@@ -1162,11 +1301,15 @@ namespace Zint {
             arg_bool(cmd, "--dotty", dotty());
         }
 
+        if (showText()) {
+            arg_bool(cmd, "--embedfont", embedVectorFont());
+        }
         arg_bool(cmd, "--esc", inputMode() & ESCAPE_MODE);
+        arg_bool(cmd, "--extraesc", inputMode() & EXTRA_ESCAPE_MODE);
         arg_bool(cmd, "--fast", inputMode() & FAST_MODE);
 
-        if (fgColor() != Qt::black) {
-            arg_color(cmd, "--fg=", fgColor());
+        if (fgStr() != QSL("000000") && fgStr() != QSL("000000FF")) {
+            arg_str(cmd, "--fg=", fgStr());
         }
 
         arg_bool(cmd, "--fullmultibyte", supportsFullMultibyte() && (option3() & 0xFF) == ZINT_FULL_MULTIBYTE);
@@ -1178,9 +1321,12 @@ namespace Zint {
             arg_bool(cmd, "--gssep", gsSep());
         }
 
-        if (isExtendable() && guardDescent() != 5.0f) {
+        if (isEANUPC() && guardDescent() != 5.0f) {
             arg_float(cmd, "--guarddescent=", guardDescent(), true /*allowZero*/);
         }
+        if (isEANUPC() && showText()) {
+            arg_bool(cmd, "--guardwhitespace", guardWhitespace());
+        }
 
         if (!autoHeight && !isFixedRatio()) {
             if (inputMode() & HEIGHTPERROW_MODE) {
@@ -1201,9 +1347,9 @@ namespace Zint {
             arg_int(cmd, "--mode=", option1());
         }
 
-        arg_bool(cmd, "--nobackground", bgColor() == QColor(0xFF, 0xFF, 0xFF, 0));
+        arg_bool(cmd, "--nobackground", nobackground);
         arg_bool(cmd, "--noquietzones", hasDefaultQuietZones() && noQuietZones());
-        arg_bool(cmd, "--notext", hasHRT() && !showText());
+        arg_bool(cmd, "--notext", notext);
         arg_data(cmd, longOptOnly ? "--output=" : "-o ", outfile, win);
 
         if (m_symbol == BARCODE_MAXICODE || isComposite()) {
@@ -1244,16 +1390,20 @@ namespace Zint {
             arg_int(cmd, "--separator=", option3());
         }
 
-        arg_bool(cmd, "--small", fontSetting() & SMALL_TEXT);
+        arg_bool(cmd, "--small", !notext && (fontSetting() & SMALL_TEXT));
 
         if (m_symbol == BARCODE_DATAMATRIX || m_symbol == BARCODE_HIBC_DM) {
-            arg_bool(cmd, "--square", option3() == DM_SQUARE);
+            arg_bool(cmd, "--square", (option3() & 0x7F) == DM_SQUARE);
         }
 
         if (supportsStructApp()) {
             arg_structapp(cmd, "--structapp=", structAppCount(), structAppIndex(), structAppID(), win);
         }
 
+        if (!notext && textGap() != 1.0f) {
+            arg_float(cmd, "--textgap=", textGap(), true /*allowZero*/);
+        }
+
         arg_bool(cmd, "--verbose", debug());
 
         if (m_symbol == BARCODE_AZTEC || m_symbol == BARCODE_HIBC_AZTEC
@@ -1299,14 +1449,6 @@ namespace Zint {
         }
     }
 
-    void QZint::arg_color(QString& cmd, const char *const opt, const QColor val) {
-        if (val.alpha() != 0xFF) {
-            cmd += QString::asprintf(" %s%02X%02X%02X%02X", opt, val.red(), val.green(), val.blue(), val.alpha());
-        } else {
-            cmd += QString::asprintf(" %s%02X%02X%02X", opt, val.red(), val.green(), val.blue());
-        }
-    }
-
     void QZint::arg_data(QString& cmd, const char *const opt, const QString& val, const bool win) {
         if (!val.isEmpty()) {
             QString text(val);
